physics = {}
flying = false

function physics.keyreleased_debug(key, _)
	if key == "kp4" then
		player.x = player.x -1
	end
	if key == "kp6" then
		player.x = player.x +1
	end
	if key == "kp8" then
		player.y = player.y -1
	end
	if key == "kp2" then
		player.y = player.y +1
	end
end

function physics.remove_solid(x, y, sizex, sizey)
	assert_num("physics.remove_solid", 4, x, y, sizex, sizey)
	for coord_x=x, x + sizex-1 do
		for coord_y=y, y + sizey-1 do
			if not buffer.physics[coord_x] then buffer.physics[coord_x] = {} end
			buffer.physics[coord_x][coord_y] = { dir = { 0, 0, 0 }, solid = false }
		end
	end
end


function physics.draw_solid(x, y, sizex, sizey)
	assert_num("physics.draw_solid", 4, x, y, sizex, sizey)
	love.graphics.setCanvas(buffer.pixel.dbg)
	if COLLISION_PRINT then
		color_push()
		love.graphics.setDefaultFilter("nearest", "nearest", 0)
		love.graphics.setBlendMode("replace")
		love.graphics.setColor(0, 0, 1, 0.4)
		love.graphics.rectangle("fill", x, y, sizex, sizey)
		love.graphics.setBlendMode("alpha")
		color_pop()
	end
	love.graphics.setCanvas()
	for coord_x=x, x + sizex-1 do
		for coord_y=y, y + sizey-1 do
			if not buffer.physics[coord_x] then buffer.physics[coord_x] = {} end
			buffer.physics[coord_x][coord_y] = { dir = { 0, 0, 0 }, solid = true }
		end
	end
end


function draw_colision(x, y, x2, y2, pixelx, pixely)
	color_lock()
	love.graphics.setLineWidth(1)
	love.graphics.setDefaultFilter("nearest", "nearest", 0)
	love.graphics.setCanvas(buffer.pixel.dbg2)
	if pixelx then
		love.graphics.setColor(.6, 0, .6, 1)
		love.graphics.rectangle("fill", x, y, x2, y2)
		love.graphics.setColor(1, 0, 0, 1)
		love.graphics.points(pixelx, pixely)
	else
		love.graphics.setColor(0, 1, 0, 1)
		love.graphics.rectangle("fill", x, y, x2, y2)
	end
	love.graphics.setCanvas()
	color_unlock()
end



function physics.is_blocked_x(x, y, sizex)
	x, y = math.floor(x), math.floor(y)
	assert_num("physics.is_blocked_x", 3, x, y, sizex)
	for coord_x=x ,x + sizex -1 do
		if buffer.physics[coord_x] and buffer.physics[coord_x][y] and buffer.physics[coord_x][y]["solid"] then
			if COLLISION_PRINT then draw_colision(x, y, sizex, 1, coord_x, y) end
			return true
		end
	end
	if COLLISION_PRINT then draw_colision(x, y, sizex, 1) end
	return false
end


function physics.is_blocked_y(x, y, sizey)
	x, y = math.floor(x), math.floor(y)
	assert_num("physics.is_blocked_y", 3, x, y, sizey)
	for coord_y=y, y + sizey -1 do
		if buffer.physics[x] and buffer.physics[x][coord_y] and buffer.physics[x][coord_y]["solid"] then
			if COLLISION_PRINT then draw_colision(x, y, 1, sizey, x, coord_y) end
			return true
		end
	end
	if COLLISION_PRINT then draw_colision(x, y, 1, sizey) end
	return false
end


function physics.keypressed( key, code, isrepeat)
	if isrepeat then return end
	if key == "p" then
		flying = not flying
		if flying then player.speed.y = 0 end
	end
end


function physics.input(dt)
	framenum = framenum + 1
	local speedModified = dt * 20
	if love.keyboard.isDown("rshift") or love.keyboard.isDown("lshift")
	then
		speedModified = speedModified + (60 * dt)
	end
	if love.keyboard.isDown("a") then
		player.speed.x = player.speed.x - speedModified
	elseif love.keyboard.isDown("d") then
		player.speed.x = player.speed.x + speedModified
	elseif love.keyboard.isDown("w") and flying then
		player.speed.y = player.speed.y - speedModified
	elseif love.keyboard.isDown("s") and flying then
		player.speed.y = player.speed.y + speedModified
	end

	if love.keyboard.isDown("space") and not flying then
		local posx = player.x + player.ava:getWidth()
		local posy = player.y + player.ava:getHeight()
		if physics.is_blocked_x(posx - player.ava:getWidth(), posy, player.ava:getWidth()) or
				player.y == world.y - player.ava:getHeight() then
			player.speed.y = -25
		end
	end
end


function physics.cap_world(newx, newy, new_sx, new_sy)
	if newx > world.x - player.ava:getWidth() then
		newx = world.x - player.ava:getWidth()
		new_sx = 0
	end
	if newx < 0 then
		newx = 0
		new_sx = 0
	end
	if newy > world.y - player.ava:getHeight() then
		newy = world.y - player.ava:getHeight()
		new_sy = 0
	end
	if newy < 0 then
		newy = 0
		new_sy = 0
	end
	return newx, newy, new_sx, new_sy
end


function physics.update(dt)
	--friction
	local new_sx = player.speed.x - player.speed.x / 16
	local new_sy
	if flying then
		new_sy = player.speed.y - player.speed.y / 16
	else
		new_sy = player.speed.y - player.speed.y / 6
	end
	--local new_sx = player.speed.x - math.sin(player.speed.x) * player.speed.x
	--local new_sy = player.speed.y - math.sin(player.speed.y / 4) * player.speed.y
	--falling
	if not flying then
		new_sy = new_sy +dt * 100
	end
	--new positions
	local futureX = player.x + player.speed.x
	local futureY = player.y + player.speed.y

	-- cap boundaries
	futureX, futureY, new_sx, new_sy = physics.cap_world(futureX, futureY, new_sx, new_sy)

	local granularity = math.sqrt(math.abs(player.speed.x * player.speed.x)+ math.abs(player.speed.y * player.speed.y)) + 1
	local stepsizeX = player.speed.x / granularity
	local stepsizeY = player.speed.y / granularity

	local currentX = player.x
	local currentY = player.y
	if not granularity or granularity < 1 then granularity = 1 end

	if flying then -- skip physics
		player.x, player.y = futureX, futureY

		player.speed.x = new_sx
		player.speed.y = new_sy
		return
	end

	do -- SCOPE
	local blocked_x, blocked_y = false, false
	for iteration=1, granularity do
		intermediateX = currentX + (stepsizeX * iteration)
		intermediateY = currentY + (stepsizeY * iteration)

		if intermediateY > futureY then -- UP
			if physics.is_blocked_x(intermediateX, intermediateY, player.ava:getWidth()) then
				blocked_y = true
			end
		end

		if intermediateY < futureY then --DOWN
			local lowerY = intermediateY + player.ava:getHeight() -1
			if physics.is_blocked_x(intermediateX, lowerY, player.ava:getWidth()) then
				blocked_y = true
				intermediateY = currentY
				-- Otherwise moving right and left on a plain surface doesn't work,
				-- this is caused by gravity, maybe a better way to do this exists? :(
			end
		end

		if intermediateX > futureX then -- ONLY LEFT
			if physics.is_blocked_y(intermediateX, intermediateY, player.ava:getHeight()) then
				blocked_x = true
			end
		end

		if intermediateX < futureX then -- ONLY RIGHT
			local lowerX = intermediateX + player.ava:getWidth() -1
			if physics.is_blocked_y(lowerX, intermediateY, player.ava:getHeight()) then
				blocked_x = true
			end
		end

		intermediateX, intermediateY, new_sx, new_sy = physics.cap_world(intermediateX, intermediateY, new_sx, new_sy)
		if blocked_x and not blocked_y then -- X blocked
			--new_sy = 0
			player.y = intermediateY
		end

		if blocked_y and not blocked_x then
			--new_sx = 0
			player.x = intermediateX
		end

		if blocked_x and blocked_y then
			break
		end

		if not blocked_x and not blocked_y then
			player.x = intermediateX
			player.y = intermediateY
		end
	end
	end -- SCOPE
	-- cap low speed
	if new_sx > -.1 and new_sx < .1 then
		new_sx = 0
	end
	if new_sy > -.1 and new_sy < .1 then
		new_sy = 0
	end
	player.speed.x = new_sx
	player.speed.y = new_sy
end
