ui = {}
ui.font, ui.bigFont = nil
if love.filesystem.getInfo("fonts/font.ttf") then
	ui.font = love.graphics.newFont("fonts/font.ttf", 20)
	ui.bigFont = love.graphics.newFont("fonts/font.ttf", 45)
else
	ui.font = love.graphics.getFont(20)
	ui.bigFont = love.graphics.getFont(45)
	print("Falling back to love2d default font")
end

ui.scale = love.graphics.newText(ui.bigFont, "scale: " .. scale)
ui.framenum = love.graphics.newText(ui.font, "framenum: " .. framenum)


function ui.update_framenum(p_framenum)
	ui.framenum = nil
	ui.framenum = love.graphics.newText(ui.font,"framenum: " .. p_framenum)
	love.graphics.setCanvas(buffer.UI)
	love.graphics.clear()
	love.graphics.setCanvas()
	ui.draw(window.x, window.y)
end


function ui.update_scale(p_scale)
	scale = p_scale
	ui.scale = nil
	ui.scale = love.graphics.newText(ui.bigFont, "scale: " .. p_scale)
	love.graphics.setCanvas(buffer.UI)
	love.graphics.clear()
	love.graphics.setCanvas()
	ui.draw(window.x, window.y)
end


local helptext = [[[P] toggle fly
[A/D] Move
[Hold Shift] Move faster
[Space] Jump
[A/S/DW] Move (flying)
[C] Clear canvas
[F] Fill foreground
[G] Fill background
[Mouse1] Draw
[Mouse2] Delete
[Hold ctrl] Draw on Background
[+/-/PGUP/PGDWN] Zoom canvas
[Cursor keys] Move camera
]]


function ui.blockDrawer(w, h)
	local blockSelector = {name = "vertical"}
	local groupEntries = love.filesystem.getDirectoryItems("textures/blocks/")
	for entry, groupName in ipairs(groupEntries) do
		local groupInfo = love.filesystem.getInfo("textures/blocks/" .. groupName)
		if groupInfo and groupInfo.type == "directory" then
		local groupList = {name = "vertical"}
		local groupEntries = love.filesystem.getDirectoryItems("textures/blocks/" .. groupName)
		for i, itemName in ipairs(groupEntries) do
			local itemList = love.filesystem.getDirectoryItems("textures/blocks/".. groupName .."/" .. itemName .. "/")
			local blockList = { name = "horizontal" }
			table.insert(groupList, { name = "label", text=itemName })
			for i2, entryName in ipairs(itemList) do
				local path = "textures/blocks/" .. groupName .. "/".. itemName .."/" .. entryName
				local newBrick = love.graphics.newImage(path)
				local NextX, nextY = 0, 0
				if i2 > 1 then
					local NextX, nextY = layout.handle(blockList, 0, 0, true)
				end
				if NextX + newBrick:getWidth() > 300 then
					table.insert(groupList, blockList)
					blockList = { name = "horizontal" }
				end
				newBrick:setFilter("nearest", "nearest")
				if drawing.currentCursorName == path then
					table.insert(blockList,
						{name = "overlayRect",
							fill = "line",
							{name = "drawTexture", -- no cursorSelect since it is the selected item
								texture = newBrick
							}
						})
				else
					table.insert(blockList,
						{name = "cursorSelect",
							identifier = path,
							kind = "picker",
							{name = "drawTexture",
								texture = newBrick
							}
						})
				end
			end
			table.insert(groupList, blockList)
		end
		table.insert(blockSelector, { name = "label", text=groupName })
		table.insert(blockSelector, { name = "overlayRect", fill = "line", groupList })
		end
	end
	return blockSelector
end


function ui.helpDrawer(w, h)
	local help = {name = "label",
		text = helptext
	}
	return help
end


function ui.tabDrawer(w, h, selectedTab)
	local drawer = {
		name = "vertical",
		{name = "rect",
			width = 200,
			height = 50,
			fill = "fill"
		},
		{name = "copySize",
			{name = "rect",
				fill = "fill",
				color = { r = .1, g = .1, b = .1 }
			},
			selectedTab
		}
	}
	return drawer
end


function ui.draw(w, h)
	window.x, window.y = w, h
	color_push()
	love.graphics.setCanvas(buffer.UI)
		uiState = {}
		love.graphics.clear( )
		local blockDrawer = ui.blockDrawer(w,h)
		layout.handle(ui.tabDrawer(window.x - 300, 0, blockDrawer), window.x - 300, 0)
		love.graphics.draw(ui.scale, 40, 10, math.pi * 10 / 360, 1, 1)
	color_pop()
	love.graphics.setCanvas()
end


function love.mousepressed(mousex, mousey)
	if mousex < window.x -300 then return end
	for i, v in ipairs(uiState) do
		if mousex >= v.x and mousex <= v.x2 then
			if mousey >= v.y and mousey <= v.y2 then
				if v.kind == "picker" then
					drawing.currentCursor = love.graphics.newImage(v.identifier)
					drawing.currentCursorName = v.identifier
					ui.draw(window.x, window.y)
					return
				elseif v.kind == "tab" then

				end
			end
		end
	end
end


ui.init = function()
	local w,h = love.window.getMode()
	ui.draw(w, h)
	ui.resize = function()
		local w,h = love.window.getMode()
		buffer.UI = love.graphics.newCanvas(w, h)
		love.graphics.setCanvas(buffer.UI)
			love.graphics.clear()
		love.graphics.setCanvas()
		ui.draw(w, h)
	end
end
